# 
#
#
#
#
import sys

import numpy as np
import pyNN.nest as sim
import cPickle as pickle
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from winnerTakeAllClass import winnerTakeAllFunctions

#
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

#
def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def connectInhib(inhibPopulation, aTemperaturePopulation):
    toInhibWt = 0.05
    fromInhibWt = 0.48
    for temp in range(0,100):
        for inhibOffset in range(0,10):
            fsa.oneNeuronStimulatesOneNeuron(aTemperaturePopulation,temp,
                                         inhibPopulation,inhibOffset,toInhibWt)
            fsa.oneNeuronInhibitsOneNeuron(inhibPopulation,inhibOffset,
                    aTemperaturePopulation,temp,fromInhibWt)

def buildHotCoffeeTopology(semanticPopulation, temperaturePopulation,
                           coffeePopulation, cokePopulation,inhibPopulation):
    connectInhib(inhibPopulation,cokePopulation)
    connectInhib(inhibPopulation,coffeePopulation)
    fsa.makeCA(semanticPopulation,COFFEE) # coffee 0
    fsa.makeCA(semanticPopulation,COKE) # coke 1
    fsa.makeCA(semanticPopulation,COLD) # cold 2
    fsa.makeCA(semanticPopulation,WARM) # warm 3
    fsa.makeCA(semanticPopulation,HOT) # hot 4

    toTempWt = 0.0015 # 0.002 too big test 7 .001 too small
    tempToSemWt = 0.004
    semToWTAWt = 0.001

    halfWTAToWTAWt = 0.01#0.02 too big
    #input temp stimulates coffee and coke temp
    for temp in range(0,100):
	fsa.oneNeuronStimulatesOneNeuron(temperaturePopulation,temp,
                                         coffeePopulation,temp,halfWTAToWTAWt)
	fsa.oneNeuronStimulatesOneNeuron(temperaturePopulation,temp,
                                         cokePopulation,temp,halfWTAToWTAWt)
        #coffee-> coffeepop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COFFEE,
                                         coffeePopulation,temp,semToWTAWt)
        #coke -> cokepop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COKE,
                                         cokePopulation,temp,semToWTAWt)

    #coffee <-> temperatures
    for temp in range(0,25):
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> cold
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COLD,tempToSemWt)
        #cold-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COLD,
                                     coffeePopulation,temp,toTempWt)

    for temp in range(25,70): 
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> warm
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,WARM,tempToSemWt)
        #warm-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,WARM,
                                     coffeePopulation,temp,toTempWt)


    for temp in range(70,100):
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> hot
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,HOT,tempToSemWt)
        #hot-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,HOT,
                                     coffeePopulation,temp,toTempWt)

    #coke <-> temperatures
    for temp in range(0,5):
        smallTempToSemWt = tempToSemWt+0.0005 #just 7 temp pop neurons
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,smallTempToSemWt)
        #cokePop -> cold  
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COLD,smallTempToSemWt)
        #cold-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COLD,
                                     cokePopulation,temp,toTempWt)

    #coke <-> temperatures

    for temp in range(5,16):
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,tempToSemWt)
        #cokePop -> warm
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,WARM,tempToSemWt)
        #warm-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,WARM,
                                     cokePopulation,temp,toTempWt)

    for temp in range(16,100):
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,tempToSemWt)
        #cokePop -> hot
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,HOT,tempToSemWt)
        #hot-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,HOT,
                                     cokePopulation,temp,toTempWt)

#--main
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 6):
	testType  = int(args[1])
	exWindow = str(args[2])
	exWeight = float(args[3])
	inWindow = str(args[4])
	inWeight = float(args[5])
#else:
#	testType = 1

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
wta = winnerTakeAllFunctions(neal,fsa)

init()

#neurons = 100
cellClass=sim.IF_cond_exp
cellParams=fsa.CELL_PARAMS
#exWindow# = #'d<1'
#inWindow# = #'d>10'
#exWeight# = #0.1
#inhibWeight# = #0.01
tempPop = wta.WTA(100,cellClass,cellParams,exWindow,inWindow,exWeight,inWeight)
tempPop.record(['spikes','v','gsyn_exc','gsyn_inh'])

#TEST
source0 = wta.createTwoInputs()
spikeSource = wta.createTwoInputs()


#test = 3
if (testType == 4):
    for i in range (25,99):
	wta.turnOnCellFromSpikeSource(spikeSource[0],tempPop,i)
if (testType == 3):
    for i in range (25,50):    
        wta.turnOnCellFromSpikeSource(spikeSource[0],tempPop,i)
elif (testType == 2):
    for i in range(5,95):
	wta.turnOnCellFromSpikeSource(spikeSource[0],tempPop,i)
elif (testType == 1):
    for i in range (47,50):    
        wta.turnOnCellFromSpikeSource(spikeSource[1],tempPop,i)

print testType

neal.nealApplyProjections()
sim.run(1000)
sim.end()

tempPopPkl = tempPop.write_data("outputs/wtab1_1000/popTemp.pkl",'spikes')
tempPop.print_v("outputs/wtab1_1000/popTempV.pkl")
tempPop.write_data("outputs/wtab1_1000/popTempE.pkl",'gsyn_exc')
tempPop.write_data("outputs/wtab1_1000/popTempI.pkl",'gsyn_inh')

